/* 
 * This file is part of the Modbus IO distribution.
 * Copyright (c) 2020 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef I_MB_CALLBACK_HPP
#define I_MB_CALLBACK_HPP

#include "mb_defs.hpp"

#include <cstdint>

//! \brief interface to handle callbacks
class i_mb_callback {
public:

	i_mb_callback(uint16_t max_address) : m_max_address(max_address){}
	
	//! \brief callback for writung In/Hold Registers and Coils
	//! \param[inout] buf 16bit value to write
	//! \param[in] address address of register/input/coil to R/W
	//! \return modbus exception code @ref mb_exc
	virtual mb_err call_write(uint16_t buf, uint16_t address) = 0;
	
	//! \brief callback for reading In/Hold Registers and Coils/Inputs
	//! \param[inout] buf pointer to 16bit value to store read value to
	//! \param[in] address address of register/input/coil to R/W
	//! \return modbus exception code @ref mb_exc
	virtual mb_err call_read(uint16_t *buf, uint16_t address) = 0;
	
	//! \brief  maximum address reachable in this callback
	uint16_t m_max_address;
};

#endif//I_MB_CALLBACK_HPP