/* 
 * This file is part of the Modbus IO distribution.
 * Copyright (c) 2020 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MODBUS_EVENTBYTES_HPP
#define MODBUS_EVENTBYTES_HPP

#include "mb_defs.hpp"

#include <cstdint>

class modbus_eventbytes {
public:	

	modbus_eventbytes();
	
	void add_recv_event_comm_err();
	void add_recv_event_char_overrun();
	void add_recv_event_listen_only();
	void add_recv_event_broadcast();
	void add_recv_event_flush(); // its a bitfield
	
	void add_sent_event(mb_exc code, bool listen_only_mode);
	
	void add_enter_listen_only();
	
	void add_inited_comm_restart();
	
	// returns elements copied to bytes 
	uint8_t get_fifo(uint8_t *bytes);
	
	void clear();

private:
	
	enum class recv_bits : uint8_t {
		communication_error	= 1 << 1,
		character_overrun	= 1 << 4,
		listen_only_mode	= 1 << 5,
		broadcast_received	= 1 << 6,
		receive_event		= 1 << 7,
	};
	enum class sent_bits : uint8_t {
		read_exception 				= 1 << 0, // Exception Codes 1-3
		slave_abort_exception 		= 1 << 1, // Exception Code  4
		slave_busy_exception 		= 1 << 2, // Exception Codes 5-6
		slave_program_nak_exception = 1 << 3, // Exception Code  7
		write_timeout_error 		= 1 << 4,
		listen_only_mode 			= 1 << 5,
		sent_event 					= 1 << 6,
	};
	const uint8_t m_entered_listen_only = 0x04;
	const uint8_t m_inited_comm_restart = 0x00;
	
	void set_fifo(uint8_t byte);
	
	uint8_t m_recv_byte;
	
	const uint8_t fifosize = 64; 
	uint8_t m_fifo[64];		//!< fifo buffer itself
	uint8_t m_fifo_wp;		//!< write pointer (write then increment)
	uint8_t m_fifo_fill;	//!< elements in fifo
};

#endif//MODBUS_EVENTBYTES_HPP