#ifndef LM75_SLEEP_HPP
#define LM75_SLEEP_HPP

#include "periph/i_pio.hpp"
#include "periph/i_i2c.hpp"
#include "periph/i_timer_poll.hpp"

#include "i_sleepable.hpp"

#include <cstdint>
#include <cstddef>

enum class lm75_err {
	SUCCESS,
	ERR_NOINIT,
	ERR_ARGS,
};

class lm75_sleep : public i_sleepable {
public:

	lm75_sleep();
	
	//! \brief init of this instance
	//! \param[in] i2c_inst i2c instance to use
	//! \param[in] pin_power_en pin for power enable
	//! \param[in] timer_inst instance for timeouts
	//! \return success or @ref lm75_err
	lm75_err init(i_i2c* i2c_inst, pio_set* pin_power_en, i_timer_poll* timer_inst);
	
	void deinit(void);
	
	//! \brief readout the device, will work async, no temperature here
	//! \return success or @ref lm75_err
	lm75_err read_device(void);
	
	//! \brief checking that readout is in progress
	//! \param[out] doing true if measure is in progress
	//! \return success or @ref lm75_err
	lm75_err read_progress(bool* doing);
	
	//! \brief reads stored variable
	//! \param[out] temp temperature with LSB = 0,1degC (-3276,7 for error)
	//! \return success or @ref lm75_err
	lm75_err read_temp(int16_t* temp);
	
	//! \brief processing internals, call in main loop
	//! \return success or @ref lm75_err
	lm75_err process(void);
	
	virtual void can_sleep(bool *can_sleep);
	
private:

	void read_and_save(void);

	enum class lm75_state {
		NoInit,
		Idle,
		Startup,
		WaitPowOn,
		Readout,
	};
	
	i_i2c* m_i2c_inst;
	pio_set* m_pin_power_en;
	i_timer_poll* m_timer_inst;
	
	lm75_state m_state;
	
	tim_poll_obj m_timer_idx;
	
	int16_t m_temp;
};

#endif//LM75_SLEEP_HPP