#ifndef CONFIG_DATA_HPP
#define CONFIG_DATA_HPP

#include "periph/eeprom.hpp"

#include "device/water_barrel.hpp"

#include "modbus\modbus.hpp"

enum class confdat_err {
	SUCCESS,
	ERR_INIT,
	ERR_ARGS,
	ERR_EXEC,
};

class config_data {
public:

	config_data();
	
	// inits and checks for empty eeprom, then writes defaults
	confdat_err init(eeprom* eeprom_inst, barrel_calibdata* barrel_calib, mb_setting* mb_settings);
	
	confdat_err write_barrel_sizes(barrel_calibdata* barrel_calib);
	
	confdat_err write_barrel_calib(barrel_calibdata* barrel_calib);
	
	confdat_err write_mb_settings(mb_setting* mb_settings);
	
	// if a eeprom cell is 0xFF the pointed data will not be touched (default data)
	confdat_err read(barrel_calibdata* barrel_calib);
	
	// if a eeprom cell is 0xFF the pointed data will not be touched (default data)
	confdat_err read(mb_setting* mb_settings);
	
private:

	// only override value if readout is not eeprom deleted cell
	void readout(uint32_t addr, void* data, uint8_t sizeee, uint8_t sizecopy, bool* errfree);
	
	eeprom* m_eeprom_inst;
};

#endif // CONFIG_DATA_HPP