#include "mb_callb_coil.hpp"

mb_callback_coils::mb_callback_coils() : 
	i_mb_callback(5),
	m_mb_inst(NULL),
	m_mb_settings(NULL),
	m_lm75_inst(NULL),
	m_wb_inst(NULL),
	m_bl_load_flag(false)
{

}

mb_err mb_callback_coils::init(modbus* mb_inst, mb_setting* mb_settings, lm75_sleep* lm75_inst, water_barrel* wb_inst, 
                               config_data* confdat_inst, barrel_calibdata* barrel_calib)
{
	m_mb_inst = NULL;
	if ((NULL == mb_inst) || (NULL == mb_settings) || (NULL == lm75_inst) || (NULL == wb_inst) || 
	    (NULL == confdat_inst) || (NULL == barrel_calib))
	{
		return mb_err::ERR_ARGS;
	}
	m_mb_inst      = mb_inst;
	m_mb_settings  = mb_settings;
	m_lm75_inst    = lm75_inst;
	m_wb_inst      = wb_inst;
	m_confdat_inst = confdat_inst;
	m_barrel_calib = barrel_calib;
	return mb_err::SUCCESS;
}

mb_err mb_callback_coils::call_write(uint16_t buf, uint16_t address) 
{
	
	barrel_err  b_err = barrel_err::SUCCESS;
	lm75_err    l_err = lm75_err::SUCCESS;
	mb_err      m_err = mb_err::SUCCESS;
	confdat_err c_err = confdat_err::SUCCESS;
	
	if (NULL == m_mb_inst)
	{
		return mb_err::ERR_INIT;
	}
	
	if (0 == buf)
	{
		// its not abortable!
		return mb_err::SUCCESS;
	}
	
	switch (address) 
	{
		case 1:
			if (buf)
			{
				m_bl_load_flag = true;
			}
			break;
		case 2: // Trig. Measure
			b_err = m_wb_inst->read_device();
			l_err = m_lm75_inst->read_device();
			break;
		case 3: // Trig. Wr. Size
			c_err = m_confdat_inst->write_barrel_sizes(m_barrel_calib);
			break;
		case 4: // Trig. Wr. Calib.
			c_err = m_confdat_inst->write_barrel_calib(m_barrel_calib);
			break;
		case 5: // Trig. Wr. modbus settings
			c_err = m_confdat_inst->write_mb_settings(m_mb_settings);
			m_mb_inst->set_address(m_mb_settings->address);
			m_err = m_mb_inst->init(m_mb_settings->baudrate, m_mb_settings->parity, false);
			break;

		default:
			return mb_err::ERR_EXEC;
	}
	
	if ((barrel_err::SUCCESS != b_err) || (lm75_err::SUCCESS != l_err) || 
	    (mb_err::SUCCESS != m_err) || (confdat_err::SUCCESS != c_err)) 
	{
		return mb_err::ERR_EXEC;
	}
	
	return mb_err::SUCCESS;
}

mb_err mb_callback_coils::call_read(uint16_t *buf, uint16_t address) 
{
	
	barrel_err b_err = barrel_err::SUCCESS;
	lm75_err   l_err = lm75_err::SUCCESS;
	
	if (NULL == m_mb_inst)
	{
		return mb_err::ERR_INIT;
	}
	
	bool boolout;
	bool temporary;
	
	switch (address) 
	{
		case 1:
			*buf = 0; // FW is running
			break;
		case 2: // Trig. Measure
			temporary = false;
			b_err = m_wb_inst->read_progress(&boolout);
			temporary |= boolout;
			l_err = m_lm75_inst->read_progress(&boolout);
			temporary |= boolout;
			*buf  = temporary;
			break;
		case 3: // Trig. Wr. Size
			*buf = 0;
			break;
		case 4: // Trig. Wr. Calib.
			*buf = 0;
			break;
		case 5: // Trig. Wr. modbus settings
			*buf = 0;
			break;

		default:
			return mb_err::ERR_EXEC;
	}
	
	if ((barrel_err::SUCCESS != b_err) || (lm75_err::SUCCESS != l_err)) 
	{
		*buf = 0;
		return mb_err::ERR_EXEC;
	}
	
	return mb_err::SUCCESS;
}

bool mb_callback_coils::bl_is_flagged(void)
{
	return m_bl_load_flag;
}