#ifndef MB_CALLBACK_COILS
#define MB_CALLBACK_COILS

#include "i_mb_callback.hpp"

#include "device/lm75_sleep.hpp"
#include "device/water_barrel.hpp"

#include "hal/config_data.hpp"

#include "modbus\modbus.hpp"

#include <cstddef>

class mb_callback_coils : public i_mb_callback {
public:

	mb_callback_coils();
	
	//! \brief init of this instance
	//! \param[in] mb_inst      instance of mpdbus to use
	//! \param[in] mb_settings  modbus settings to set
	//! \param[in] lm75_inst    instance of lm75 to use
	//! \param[in] wb_inst      instance of water barrel to use
	//! \param[in] confdat_inst instance of nvmem to use
	//! \param[in] barrel_calib data of barrel data to store
	//! \return modbus exception code @ref mb_exc
	mb_err init(modbus* mb_inst, mb_setting* mb_settings, lm75_sleep* lm75_inst, water_barrel* wb_inst, 
	            config_data* confdat_inst, barrel_calibdata* barrel_calib);
	
	//! \brief callback for R/W In/Hold Registers
	//! \param[inout] buf 16bit value do R/W
	//! \param[in] address address of register to R/W
	//! \return modbus exception code @ref mb_exc
	virtual mb_err call_write(uint16_t buf, uint16_t address);
	virtual mb_err call_read(uint16_t *buf, uint16_t address);
	
	//! \brief polls that bl is wanted or not
	//! \return true -> jump to bootloader
	bool bl_is_flagged(void);
	
private:
	
	modbus*           m_mb_inst;
	mb_setting*       m_mb_settings;
	lm75_sleep*       m_lm75_inst;
	water_barrel*     m_wb_inst;
	config_data*      m_confdat_inst;
	barrel_calibdata* m_barrel_calib;
	
	//! \brief true -> jump to bootloader
	bool m_bl_load_flag;
};

#endif//MB_CALLBACK_COILS