#include "mb_callb_hold.hpp"

#include "cmsis_includer.h"

#include "modbus\modbus_reg_helper.hpp"

#include <cstring>

mb_callback_hold::mb_callback_hold() : 
	i_mb_callback(1002),
	m_barrel_calib(NULL),
	m_mb_inst(NULL),
	m_mb_settings(NULL)
{
}

mb_err mb_callback_hold::init(barrel_calibdata* barrel_calib, modbus* mb_inst, mb_setting* mb_settings) 
{
	m_barrel_calib = NULL;
	if ((NULL == barrel_calib) || (NULL == mb_inst) || (NULL == mb_settings))
	{
		return mb_err::ERR_ARGS;
	}
	
	m_barrel_calib = barrel_calib;
	m_mb_inst      = mb_inst;
	m_mb_settings  = mb_settings;
	
	return mb_err::SUCCESS;
}

mb_err mb_callback_hold::call_write(uint16_t buf, uint16_t address) 
{
	if (NULL == m_barrel_calib)
	{
		return mb_err::ERR_INIT;
	}
	
	mb_err err;
	
	switch (address) {
		
		case 1: // WB Offset LO
			mb_reg_help::set_16b_in_32b(0, buf, &m_barrel_calib->pegel_offset);
			break;
		case 2: // WB Offset HI
			mb_reg_help::set_16b_in_32b(1, buf, &m_barrel_calib->pegel_offset);
			break;
		case 3: // WB dia_bottom LO
			mb_reg_help::set_16b_in_32b(0, buf, &m_barrel_calib->barrel_dia_bottom);
			break;
		case 4: // WB dia_bottom HI
			mb_reg_help::set_16b_in_32b(1, buf, &m_barrel_calib->barrel_dia_bottom);
			break;
		case 5: // WB dia_top LO
			mb_reg_help::set_16b_in_32b(0, buf, &m_barrel_calib->barrel_dia_top);
			break;
		case 6: // WB dia_top HI
			mb_reg_help::set_16b_in_32b(1, buf, &m_barrel_calib->barrel_dia_top);
			break;
		case 7: // WB height LO
			mb_reg_help::set_16b_in_32b(0, buf, &m_barrel_calib->barrel_height);
			break;
		case 8: // WB height HI
			mb_reg_help::set_16b_in_32b(1, buf, &m_barrel_calib->barrel_height);
			break;
			
		case 9:  // WB Poly[0] LO
			mb_reg_help::set_16b_in_float(0, buf, &m_barrel_calib->poly[0]);
			break;
		case 10: // WB Poly[0] HI
			mb_reg_help::set_16b_in_float(1, buf, &m_barrel_calib->poly[0]);
			break;
		case 11: // WB Poly[1] LO
			mb_reg_help::set_16b_in_float(0, buf, &m_barrel_calib->poly[1]);
			break;
		case 12: // WB Poly[1] HI
			mb_reg_help::set_16b_in_float(1, buf, &m_barrel_calib->poly[1]);
			break;
		case 13: // WB Poly[2] LO
			mb_reg_help::set_16b_in_float(0, buf, &m_barrel_calib->poly[2]);
			break;
		case 14: // WB Poly[2] HI
			mb_reg_help::set_16b_in_float(1, buf, &m_barrel_calib->poly[2]);
			break;
		
		case 15: // WB adc_val_zero Sensor
			m_barrel_calib->adc_val_zero[ARR_IDX_SENS] = buf;
			break;
		case 16: // WB adc_val_zero VCC
			m_barrel_calib->adc_val_zero[ARR_IDX_VCC]  = buf;
			break;
		case 17: // WB adc_val_full Sensor
			m_barrel_calib->adc_val_full[ARR_IDX_SENS] = buf;
			break;
		case 18: // WB adc_val_full VCC
			m_barrel_calib->adc_val_full[ARR_IDX_VCC]  = buf;
			break;
			
		case 1000:
			err = m_mb_inst->check_address(buf);
			if (mb_err::SUCCESS == err)
			{
				m_mb_settings->address = buf;
			}
			return err;
		case 1001:
			return m_mb_inst->check_baud(&m_mb_settings->baudrate, buf);
		case 1002:
			return m_mb_inst->check_parity(&m_mb_settings->parity, buf);
		default:
			return mb_err::ERR_ARGS;
	}
	
	return mb_err::SUCCESS;
}
	
mb_err mb_callback_hold::call_read(uint16_t *buf, uint16_t address) 
{
	if (NULL == m_barrel_calib)
	{
		return mb_err::ERR_INIT;
	}
	
	switch (address) {
		
		case 1: // WB Offset LO
			mb_reg_help::get_16b_in_32b(0, buf, m_barrel_calib->pegel_offset);
			break;
		case 2: // WB Offset HI
			mb_reg_help::get_16b_in_32b(1, buf, m_barrel_calib->pegel_offset);
			break;
		case 3: // WB dia_bottom LO
			mb_reg_help::get_16b_in_32b(0, buf, m_barrel_calib->barrel_dia_bottom);
			break;
		case 4: // WB dia_bottom HI
			mb_reg_help::get_16b_in_32b(1, buf, m_barrel_calib->barrel_dia_bottom);
			break;
		case 5: // WB dia_top LO
			mb_reg_help::get_16b_in_32b(0, buf, m_barrel_calib->barrel_dia_top);
			break;
		case 6: // WB dia_top HI
			mb_reg_help::get_16b_in_32b(1, buf, m_barrel_calib->barrel_dia_top);
			break;
		case 7: // WB height LO
			mb_reg_help::get_16b_in_32b(0, buf, m_barrel_calib->barrel_height);
			break;
		case 8: // WB height HI
			mb_reg_help::get_16b_in_32b(1, buf, m_barrel_calib->barrel_height);
			break;
			
		case 9:  // WB Poly[0] LO
			mb_reg_help::get_16b_in_float(0, buf, m_barrel_calib->poly[0]);
			break;
		case 10: // WB Poly[0] HI
			mb_reg_help::get_16b_in_float(1, buf, m_barrel_calib->poly[0]);
			break;
		case 11: // WB Poly[1] LO
			mb_reg_help::get_16b_in_float(0, buf, m_barrel_calib->poly[1]);
			break;
		case 12: // WB Poly[1] HI
			mb_reg_help::get_16b_in_float(1, buf, m_barrel_calib->poly[1]);
			break;
		case 13: // WB Poly[2] LO
			mb_reg_help::get_16b_in_float(0, buf, m_barrel_calib->poly[2]);
			break;
		case 14: // WB Poly[2] HI
			mb_reg_help::get_16b_in_float(1, buf, m_barrel_calib->poly[2]);
			break;
		
		case 15: // WB adc_val_zero Sensor
			*buf = m_barrel_calib->adc_val_zero[ARR_IDX_SENS];
			break;
		case 16: // WB adc_val_zero VCC
			*buf = m_barrel_calib->adc_val_zero[ARR_IDX_VCC];
			break;
		case 17: // WB adc_val_full Sensor
			*buf = m_barrel_calib->adc_val_full[ARR_IDX_SENS];
			break;
		case 18: // WB adc_val_full VCC
			*buf = m_barrel_calib->adc_val_full[ARR_IDX_VCC];
			break;
			
		case 1000:
			*buf = m_mb_settings->address;
			break;
		case 1001:
			*buf = (uint16_t)m_mb_settings->baudrate;
			break;
		case 1002:
			*buf = (uint16_t)m_mb_settings->parity;
			break;
		default:
			return mb_err::ERR_ARGS;
	}
	
	return mb_err::SUCCESS;	
}