#include "mb_callb_inreg.hpp"

mb_callback_inreg::mb_callback_inreg() : 
	i_mb_callback(9),
	m_lm75_inst(NULL),
	m_wb_inst(NULL)
{

}

mb_err mb_callback_inreg::init(lm75_sleep* lm75_inst, water_barrel* wb_inst)
{
	m_lm75_inst = lm75_inst;
	m_wb_inst   = wb_inst;
	return mb_err::SUCCESS;
}

mb_err mb_callback_inreg::call_write(uint16_t buf, uint16_t address) 
{
	(void)buf;
	(void)address;
	return mb_err::ERR_EXEC;
}

mb_err mb_callback_inreg::call_read(uint16_t *buf, uint16_t address) 
{
	
	barrel_err b_err = barrel_err::SUCCESS;
	lm75_err   l_err = lm75_err::SUCCESS;
	
	static uint32_t temporary;
	
	if ((NULL == m_lm75_inst) || (NULL == m_wb_inst))
	{
		return mb_err::ERR_EXEC;
	}
	
	switch (address) 
	{
		case 1: // water level mm LO
			b_err = m_wb_inst->read_level(&temporary);
			*buf  = temporary;
			break;
		case 2: // water level mm HI
			*buf  = temporary >> 16;
			break;
		case 3: // water volume ml LO
			b_err = m_wb_inst->read_volume(&temporary);
			*buf  = temporary;
			break;
		case 4: // water volume ml HI
			*buf  = temporary >> 16;
			break;
		case 5: // temperature in LSB = 0,1degC
			l_err = m_lm75_inst->read_temp((int16_t*)buf);
			break;
		case 6: // sensor voltage in mV
			b_err = m_wb_inst->read_volts(wb_readout::Sensor, buf);
			break;
		case 7: // vcc voltage in mV
			b_err = m_wb_inst->read_volts(wb_readout::VCC, buf);
			break;
		case 8: // sensor adc val
			b_err = m_wb_inst->read_adc(wb_readout::Sensor, buf);
			break;
		case 9: // vcc adc val
			b_err = m_wb_inst->read_adc(wb_readout::VCC, buf);
			break;
		default:
			return mb_err::ERR_EXEC;
	}
	
	if ((barrel_err::SUCCESS != b_err) && (lm75_err::SUCCESS != l_err)) 
	{
		*buf      = 0;
		temporary = 0;
		return mb_err::ERR_EXEC;
	}
	
	return mb_err::SUCCESS;
}