#ifndef MB_CALLBACK_INREG
#define MB_CALLBACK_INREG

#include "i_mb_callback.hpp"

#include "device/lm75_sleep.hpp"
#include "device/water_barrel.hpp"

#include <cstddef>

class mb_callback_inreg : public i_mb_callback {
public:

	mb_callback_inreg();
	
	//! \brief init of this instance
	//! \param[in] lm75_inst instance of lm75 to use
	//! \param[in] wb_inst   instance of water barrel to use
	//! \return modbus exception code @ref mb_exc
	mb_err init(lm75_sleep* lm75_inst, water_barrel* wb_inst);
	
	//! \brief callback for writung In/Hold Registers and Coils
	//! \param[inout] buf 16bit value to write
	//! \param[in] address address of register/input/coil to R/W
	//! \return modbus exception code @ref mb_exc
	virtual mb_err call_write(uint16_t buf, uint16_t address);
	
	//! \brief callback for reading In/Hold Registers and Coils/Inputs
	//! \param[inout] buf pointer to 16bit value to store read value to
	//! \param[in] address address of register/input/coil to R/W
	//! \return modbus exception code @ref mb_exc
	virtual mb_err call_read(uint16_t *buf, uint16_t address);
	
private:

	lm75_sleep*   m_lm75_inst;
	water_barrel* m_wb_inst;
};

#endif//MB_CALLBACK_INREG