#ifndef I2C_L4_HPP
#define I2C_L4_HPP

#include <cstdint>

#include "i_i2c.hpp"

#include "cmsis_includer.h"

class i2c_l4 : public i_i2c {
public:
	
	i2c_l4();
	
	//! \brief writes data to slave
	//! \param i2c_in i2c periph to use
	//! \param clock_bus clock in Hz of the I2C APB bus
	//! \param clock_i2c wanted clock on the i2c bus
	//! \return @ref i2c_err
	virtual i2c_err init(I2C_TypeDef *i2c_in, uint32_t clock_bus, uint32_t clock_i2c);
	
	void deinit(void);
	
	//! \brief writes data to slave and sends start + address before
	//! \param address address of slave device (left justified)
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err write(uint8_t address, uint8_t *data, uint32_t len, bool stopafter);
	
	//! \brief writes data to slave and sends start + address + command before
	//! \param address address of slave device (left justified)
	//! \param command commandbytes to send
	//! \param clen len of command to send
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err write(uint8_t address, uint8_t *command, uint8_t clen, uint8_t *data, uint32_t len, bool stopafter);
	
	//! \brief reads data from slave and sends start + address + command before
	//! \param address address of slave device (left justified)
	//! \param command commandbytes to send
	//! \param clen len of command to send
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err read(uint8_t address, uint8_t *command, uint8_t clen, uint8_t *data, uint32_t len, bool stopafter);
	

private:

	enum class i2cdir {
		send,
		receive,
	};

	virtual i2c_err write_chunk(uint8_t *data, uint32_t *chunksize, bool start, bool stopafter, bool reload);
	
	virtual i2c_err read_chunk(uint8_t *data, uint32_t *chunksize, bool start, bool stopafter, bool reload);
	
	virtual uint32_t start_chunk(uint32_t *chunksize, bool reload);
	
	virtual i2c_err setaddress(uint8_t address, i2cdir dir, bool stopafter);
	
	virtual i2c_err sentpoll();
	
	virtual i2c_err receivedpoll();
	
	I2C_TypeDef *i2c;
};

#endif//I2C_L4_HPP