#ifndef I_I2C_HPP
#define I_I2C_HPP

#include <cstdint>

#define IIC_READ	1
#define IIC_WRITE	0

enum class i2c_err {
	SUCCESS,
	NACKED,		// the slave nacked
};

class i_i2c {
public:
	
	//! \brief writes data to slave and sends start + address before
	//! \param address address of slave device (left justified)
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err write(uint8_t address, uint8_t *data, uint32_t len, bool stopafter) = 0;
	
	//! \brief writes data to slave and sends start + address + command before
	//! \param address address of slave device (left justified)
	//! \param command commandbytes to send
	//! \param clen len of command to send
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err write(uint8_t address, uint8_t *command, uint8_t clen, uint8_t *data, uint32_t len, bool stopafter) = 0;
	
	//! \brief reads data from slave and sends start + address + command before
	//! \param address address of slave device (left justified)
	//! \param command commandbytes to send
	//! \param clen len of command to send
	//! \param data data to send
	//! \param len len of data to send
	//! \param stopafter generate a stop after sending (for restart)
	//! \return @ref i2c_err
	virtual i2c_err read(uint8_t address, uint8_t *command, uint8_t clen, uint8_t *data, uint32_t len, bool stopafter) = 0;
};


#endif//I_I2C_HPP