#ifndef I_TIMER_IRQ_HPP
#define I_TIMER_IRQ_HPP

#include <cstdint>

// a timer interface for a timer that throws an irq after a settable time

enum class timer_irq_err {
	SUCCESS,
	ERR_ARGS,
	ERR_INIT,
};

//! \brief mb uart callback
//! \param[in] arg userarg given
typedef void (*timer_irq_callb)(void*);

class i_timer_irq {
public:
	
	//! \brief init a list of pins
	//! \param us to set time in us
	//! \return @ref timer_irq_err
	virtual timer_irq_err set_time(uint32_t us) = 0;
	
	//! \brief (re)starts a timer
	//! \return @ref timer_irq_err
	virtual timer_irq_err restart() = 0;
	
	//! \brief stops a timer
	//! \return @ref timer_irq_err
	virtual timer_irq_err stop() = 0;
	
	//! \brief reads a single pin
	//! \param callb callback to set (NULL for deleting)
	//! \param arg arg given to callback (NULL allowed)
	//! \return @ref timer_irq_err
	virtual timer_irq_err set_callback(timer_irq_callb callb, void* arg) = 0;
};

#endif // I_TIMER_IRQ_HPP