#include "timer_poll_base.hpp"

#include <cstddef>

timer_poll_base::timer_poll_base() :
	m_config(NULL),
	m_debug_pin(NULL)
{
}
	
timer_poll_err timer_poll_base::init(const timer_poll_base_config* config, pio_set* debug_pin)
{
	if (NULL != m_config)
	{	
		m_config->timer->CR1 = 0;
	}
	m_config = NULL;
	
	if ((NULL == config) || (NULL == config->timer))
	{
		return timer_poll_err::ERR_ARGS;
	}
	
	// TODO: RCC
	uint32_t timer_psc  = 1*16000000/1000; // PSC for 1ms
	         timer_psc -= 1;
	
	TIM_TypeDef *tim = config->timer;
	tim->CR1  = 0;
	tim->CR2  = 0;
	tim->SMCR = 0;
	tim->DIER = 0;
	tim->SR   = 0;
	
	tim->ARR  = 0xFFFF;
	tim->PSC  = timer_psc;
	
	// setting UIF for updating shadow registers
	tim->EGR = TIM_EGR_UG;
	tim->SR  = 0;
	
	tim->CR1 |= TIM_CR1_CEN;
	
	m_config    = config;
	m_debug_pin = debug_pin;
	return timer_poll_err::SUCCESS;
}
	
timer_poll_err timer_poll_base::start(tim_poll_obj* idx, uint32_t time_ms)
{
	if (NULL == m_config)
	{
		return timer_poll_err::ERR_INIT;
	}
	else if ((NULL == idx) || (time_ms > 10000))
	{
		return timer_poll_err::ERR_ARGS;
	}
	
	TIM_TypeDef *tim = m_config->timer;

	idx->startval = tim->CNT;
	idx->time_ms  = time_ms;
	
	if (m_debug_pin)
	{
		m_debug_pin->enable();
	}
	
	return timer_poll_err::SUCCESS;
}
	
timer_poll_err timer_poll_base::check_expiry(tim_poll_obj* idx, bool* expired)
{
	if (NULL == m_config)
	{
		return timer_poll_err::ERR_INIT;
	}
	else if ((NULL == idx) || (NULL == expired))
	{
		return timer_poll_err::ERR_ARGS;
	}
	
	TIM_TypeDef *tim = m_config->timer;
	
	uint32_t time_now = tim->CNT;
	uint32_t time_diff;
	
	// overflowed?
	if (time_now >= idx->startval)
	{
		time_diff = time_now - idx->startval;
	}
	else
	{
		time_diff = (tim->ARR - idx->startval) + time_now;
	}
	
	*expired = false;
	if (time_diff >= idx->time_ms)
	{
		*expired = true;
		if (m_debug_pin)
		{
			m_debug_pin->disable();
		}
	}
	
	return timer_poll_err::SUCCESS;
}