#ifndef TIMER_POLL_BASE_HPP
#define TIMER_POLL_BASE_HPP

#include <cstdint>

#include "cmsis_includer.h"
#include "i_timer_poll.hpp"
#include "i_pio.hpp"

struct timer_poll_base_config
{
	TIM_TypeDef *timer;	 //!< timer instance
};


class timer_poll_base : public i_timer_poll {
public:

	timer_poll_base();
	
	//! \brief init timer irq instance
	//! \param config config for timer
	//! \param debug_pin debug pin for seeing timings (H for timer duration), (NULL disables)
	//! \return @ref timer_irq_err
	virtual timer_poll_err init(const timer_poll_base_config* config, pio_set* debug_pin);
	
	//! \brief starts a timer obj
	//! \param idx object handle
	//! \param time_ms time in m2 to wait
	//! \return @ref timer_poll_err
	virtual timer_poll_err start(tim_poll_obj* idx, uint32_t time_ms);
	
	//! \brief checks if time is over
	//! \param idx object handle
	//! \param expired expired or not
	//! \return @ref timer_poll_err
	virtual timer_poll_err check_expiry(tim_poll_obj* idx, bool* expired);
	
private:
	
	//! \brief config for this instance
	const timer_poll_base_config* m_config;
	
	pio_set* m_debug_pin;
	
};

#endif // TIMER_POLL_BASE_HPP