/* 
 * This file is part of the Modbus IO distribution.
 * Copyright (c) 2020 Martin Wende.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

ENTRY(Reset_Handler)
SECTIONS{

	/* start of uC flash */
	. = 0x08000000;
	_flash_start = .;
	
	/* the program has to be in the flash */
	.text : {
		KEEP(*(.text.vectors)) /* NVIC vectors have to be at the start or nothing works */
		KEEP(*(.text.handler)) /* irq vectirs here with keep or theyre discared with lto flagged cause of weak */
		*(.text*)              /* all other code */
	}
	
	/* readonly things (= constants) also in flash */
	. = ALIGN(4);
	.rodata : {
		*(.rodata*)
	}
	
	/* array with function pointers of (static) cpp constructors */
	.cppinit : {
		. = ALIGN(4);
		_init_array_start_ = .;
		KEEP(*(.init_array))
		_init_array_end_ = .;
	}
	
	/* here lies .data in the flash to be copied at startup */
	. = ALIGN(4);
	_rom_data_start = .;
	_rom_data2_start = . + _data_size;
   
	/* to detect if the program is to big for the flash */
	_flash_end = .;
	
	
	
	/* start of ram */
	. = 0x20000000;
	
	/* .data is linked to here, but saved in the flash */
	.data : AT( _rom_data_start ) {
		_data_start = .;
		*(.data*)
		_data_end = .;
	}
	_data_size = _data_end - _data_start;
	
	. = ALIGN(4);
	.bss : {
		_bss_start = .;
		*(.bss*)
		*(COMMON*)
		*(.COMMON*)
		_bss_end = .;
	}
	_bss_size = _bss_end - _bss_start;
	
	/* after the program with its static data follows the heap with dynamic data */
	. = ALIGN(4);
	end = .;
	
	/* stack to the end of the ram */
	_estack = 0x20000000 + 0x2000;
	
	
	/* checks */
	ASSERT(_flash_end - _flash_start < 0x10000, "Flash to small!")
	ASSERT(_data_size + _bss_size < 0x2000, "RAM to small!")
}